Package: episomer
Title: Early Detection of Public Health Threats from Social Media Data
Version: 3.0.35
Authors@R: c( 
     person(given = "Laura",
            family = "Espinosa",
            role = c("aut", "fnd", "cre"),
            email = "laura.espinosa@ecdc.europa.eu",
            comment = c(ORCID = "0000-0003-0748-9657", "Project manager, author of the design and concept of the package, and package maintainer")),
     person(given = "Francisco",
            family = "Orchard",
            role = c("aut", "ctr"),
            email = "f.orchard@epiconcept.fr",
            comment = c(ORCID = "0000-0001-5793-3301", "Author of the package and original code")),
     person(given = "Gianfranco",
            family = "Spiteri",
            role = c("ctb", "fnd"),
            email = "gianfranco.spiteri@ecdc.europa.eu"),
     person(given = "Ariana",
            family = "Wijermans",
            role = "ctb",
            email = "ariana.wijermans@ecdc.europa.eu",
            comment = "Contributor to the design and concept of the original version of the package"),
     person(given = "Thomas",
            family = "Mollet",
            role = c("ctb"),
            email = "thomas.mollet@ecdc.europa.eu",
            comment = "Contributor to the design and concept of the original version of the package"),
     person(given = "Adrian",
            family = "Prodan",
            role = "ctb",
            email = "adrian.prodan@ecdc.europa.eu"),
     person(given = "Martin",
            family = "Norling",
            role = "ctb",
            email = "martin.norling@ecdc.europa.eu"),
     person(given = "Enrique",
            family = "Delgado",
            role = "ctb",
            email = "enrique.delgado@ecdc.europa.eu"),
     person(given = "Thomas",
            family = "Czernichow",
            role = "ctb",
            email = "thomas.czernichow@epiconcept.fr"),
     person(given = "Maria",
            family = "Prieto Gonzalez",
            role = "ctb",
            email = "m.gonzalez@epiconcept.fr"),
     person(given = "Esther",
            family = "Kissling",
            role = "ctb",
            email = "e.kissling@epiconcept.fr"), 
     person(given = "Michael",
            family = "Höhle",
            role = "ctb",
            email = "hoehle@math.su.se"),
     person(given = "Yohann",
            family = "Mansiaux",
            role = "ctb",
            email = "y.mansiaux@epiconcept.fr")
     )
Description: It allows you to automatically monitor trends of social media messages by time, place and topic aiming at detecting public health threats early through the detection of signals (i.e., an unusual increase in the number of messages per time, topic and location). It was designed to focus on infectious diseases, and it can be extended to all hazards or other fields of study by modifying the topics and keywords. More information on the original package 'epitweetr' is available in the peer-review publication Espinosa et al. (2022) <doi:10.2807/1560-7917.ES.2022.27.39.2200177>.
License: EUPL
URL: https://github.com/EU-ECDC/episomer
BugReports: https://github.com/EU-ECDC/episomer/issues
Encoding: UTF-8
Imports: dplyr, curl, DT, emayili, future, httr, htmltools, jsonlite,
        keyring, ggplot2, janitor, magrittr, parallel, plotly,
        processx, readxl, rlang, rmarkdown, rnaturalearthdata,
        openxlsx, shiny, stringr, stats, tibble, tools, utils, xtable,
        httr2, lubridate, sf, cli
RoxygenNote: 7.3.3
Suggests: knitr, taskscheduleR, testthat (>= 3.0.0)
VignetteBuilder: knitr
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-05-08 06:30:49 UTC; fod
Author: Laura Espinosa [aut, fnd, cre] (ORCID:
    <https://orcid.org/0000-0003-0748-9657>, Project manager, author of
    the design and concept of the package, and package maintainer),
  Francisco Orchard [aut, ctr] (ORCID:
    <https://orcid.org/0000-0001-5793-3301>, Author of the package and
    original code),
  Gianfranco Spiteri [ctb, fnd],
  Ariana Wijermans [ctb] (Contributor to the design and concept of the
    original version of the package),
  Thomas Mollet [ctb] (Contributor to the design and concept of the
    original version of the package),
  Adrian Prodan [ctb],
  Martin Norling [ctb],
  Enrique Delgado [ctb],
  Thomas Czernichow [ctb],
  Maria Prieto Gonzalez [ctb],
  Esther Kissling [ctb],
  Michael Höhle [ctb],
  Yohann Mansiaux [ctb]
Maintainer: Laura Espinosa <laura.espinosa@ecdc.europa.eu>
Repository: CRAN
Date/Publication: 2026-05-08 10:50:02 UTC
Built: R 4.5.2; ; 2026-05-08 12:49:36 UTC; unix
