# Generated by roxygen2: do not edit by hand

export("%>%")
export(add_new_social_media)
export(admin_app)
export(bluesky_create_session)
export(calculate_region_alerts)
export(calculate_regions_alerts)
export(check_all)
export(create_map)
export(create_snapshot)
export(create_topchart)
export(create_topwords)
export(dashboard_app)
export(detect_loop)
export(download_dependencies)
export(ears_t_reweighted)
export(episomer_app)
export(fs_loop)
export(generate_alerts)
export(geolocate_text)
export(get_aggregates)
export(get_alerts)
export(get_tasks)
export(health_check)
export(is_detect_running)
export(is_fs_running)
export(is_search_running)
export(missing_search_jobs)
export(register_detect_runner_task)
export(register_fs_monitor)
export(register_fs_runner_task)
export(register_search_runner_task)
export(save_config)
export(search_loop)
export(search_loop_worker)
export(search_posts)
export(setup_config)
export(sm_api_get_token_bluesky)
export(sm_api_search_bluesky)
export(sm_api_set_auth_bluesky)
export(stop_detect_runner_task)
export(stop_fs_runner_task)
export(stop_search_runner_task)
export(trend_line)
export(update_geonames)
export(update_languages)
importFrom(DT,dataTableOutput)
importFrom(DT,dataTableProxy)
importFrom(DT,datatable)
importFrom(DT,formatPercentage)
importFrom(DT,renderDataTable)
importFrom(DT,replaceData)
importFrom(dplyr,arrange)
importFrom(dplyr,bind_rows)
importFrom(dplyr,dense_rank)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,if_else)
importFrom(dplyr,mutate)
importFrom(dplyr,rename)
importFrom(dplyr,select)
importFrom(dplyr,summarize)
importFrom(dplyr,ungroup)
importFrom(future,future)
importFrom(future,multisession)
importFrom(future,plan)
importFrom(ggplot2,aes)
importFrom(ggplot2,coord_fixed)
importFrom(ggplot2,coord_flip)
importFrom(ggplot2,coord_sf)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,element_text)
importFrom(ggplot2,fortify)
importFrom(ggplot2,geom_col)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_polygon)
importFrom(ggplot2,geom_sf)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggsave)
importFrom(ggplot2,labs)
importFrom(ggplot2,margin)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_size_continuous)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_classic)
importFrom(ggplot2,xlab)
importFrom(grDevices,png)
importFrom(httr2,is_online)
importFrom(httr2,last_response)
importFrom(httr2,req_body_json)
importFrom(httr2,req_error)
importFrom(httr2,req_headers)
importFrom(httr2,req_perform)
importFrom(httr2,req_retry)
importFrom(httr2,req_url_query)
importFrom(httr2,request)
importFrom(httr2,resp_body_json)
importFrom(httr2,resp_check_status)
importFrom(httr2,resp_status)
importFrom(jsonlite,fromJSON)
importFrom(jsonlite,rbind_pages)
importFrom(jsonlite,read_json)
importFrom(jsonlite,stream_in)
importFrom(lubridate,as_datetime)
importFrom(magrittr,"%>%")
importFrom(magrittr,`%>%`)
importFrom(plotly,config)
importFrom(plotly,ggplotly)
importFrom(plotly,layout)
importFrom(plotly,plotlyOutput)
importFrom(plotly,renderPlotly)
importFrom(readxl,read_excel)
importFrom(rlang,current_env)
importFrom(rmarkdown,render)
importFrom(sf,st_as_sf)
importFrom(sf,st_crs)
importFrom(sf,st_transform)
importFrom(shiny,actionButton)
importFrom(shiny,checkboxInput)
importFrom(shiny,column)
importFrom(shiny,conditionalPanel)
importFrom(shiny,dateRangeInput)
importFrom(shiny,downloadButton)
importFrom(shiny,downloadHandler)
importFrom(shiny,fileInput)
importFrom(shiny,fluidPage)
importFrom(shiny,fluidRow)
importFrom(shiny,h2)
importFrom(shiny,h3)
importFrom(shiny,h4)
importFrom(shiny,h5)
importFrom(shiny,htmlOutput)
importFrom(shiny,invalidateLater)
importFrom(shiny,navbarPage)
importFrom(shiny,need)
importFrom(shiny,numericInput)
importFrom(shiny,observe)
importFrom(shiny,observeEvent)
importFrom(shiny,passwordInput)
importFrom(shiny,radioButtons)
importFrom(shiny,renderText)
importFrom(shiny,renderUI)
importFrom(shiny,selectInput)
importFrom(shiny,shinyApp)
importFrom(shiny,sliderInput)
importFrom(shiny,span)
importFrom(shiny,tabPanel)
importFrom(shiny,textAreaInput)
importFrom(shiny,textInput)
importFrom(shiny,uiOutput)
importFrom(shiny,updateDateRangeInput)
importFrom(shiny,updateSliderInput)
importFrom(shiny,validate)
importFrom(stats,qt)
importFrom(stats,reorder)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stringr,str_replace_all)
importFrom(tools,md5sum)
importFrom(utils,URLencode)
importFrom(utils,browseURL)
importFrom(utils,download.file)
importFrom(utils,head)
importFrom(utils,tail)
importFrom(utils,unzip)
importFrom(utils,write.csv)
