## ----echo = FALSE-------------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

library(episomer)
package_name <- "episomer"
package_desc <- packageDescription(package_name)
pkg_version <- package_desc$Version
pkg_date <- package_desc$Date
authors_str <- gsub("^ *|(?<= ) |\n| *$", "", package_desc$Authors, perl = TRUE)
author_list <- eval(parse(text = authors_str))
pkg_authors <- paste(
  format(
    author_list,
    include = c("given", "family", "email", "comment"),
    braces = list(
      email = c("<", ">,<br />"),
      comment = c("", "")
    )
  ),
  collapse = "<br /><br />"
)
pkg_maintainer <- package_desc$Maintainer
pkg_license <- package_desc$License
pkg_url <- package_desc$URL

## ----echo=FALSE, fig.cap="", out.width = '100%'-------------------------------
knitr::include_graphics("https://raw.githubusercontent.com/EU-ECDC/episomer/main/img/dashboard.png")

## ----echo=FALSE, fig.cap="", out.width = '100%'-------------------------------
knitr::include_graphics("https://raw.githubusercontent.com/EU-ECDC/episomer/main/img/alerts.png")

## ----echo=FALSE, fig.cap="", out.width = '100%'-------------------------------
knitr::include_graphics("https://raw.githubusercontent.com/EU-ECDC/episomer/main/img/geotag_evaluation.png")

## ----echo=FALSE, fig.cap="", out.width = '100%'-------------------------------
knitr::include_graphics("https://raw.githubusercontent.com/EU-ECDC/episomer/main/img/data_protection.png")

## ----echo=FALSE, fig.cap="", out.width = '100%'-------------------------------
knitr::include_graphics("https://raw.githubusercontent.com/EU-ECDC/episomer/main/img/config.png")

## ----echo=FALSE, fig.cap="", out.width = '100%'-------------------------------
knitr::include_graphics("https://raw.githubusercontent.com/EU-ECDC/episomer/main/img/troubleshoot.png")

## ----eval=FALSE, echo=TRUE----------------------------------------------------
# install.packages(episomer)

## ----eval=FALSE, echo=TRUE----------------------------------------------------
# system("java -version")

## ----eval=FALSE, echo=TRUE----------------------------------------------------
# Sys.getenv("JAVA_HOME")

## ----eval=FALSE, echo=TRUE----------------------------------------------------
# library(episomer)
# episomer_app("data_dir")

## ----eval=FALSE, echo=TRUE----------------------------------------------------
# library(episomer)
# episomer_app("data_dir")

## ----echo=FALSE, fig.cap="", out.width = '50%'--------------------------------
knitr::include_graphics("https://raw.githubusercontent.com/EU-ECDC/episomer/main/img/run_tasks.png")

## ----eval=FALSE, echo=TRUE----------------------------------------------------
# library(episomer)
# fs_loop("data_dir")

## ----eval=FALSE, echo=TRUE----------------------------------------------------
# library(episomer)
# search_loop("data_dir")

## ----eval=FALSE, echo=TRUE----------------------------------------------------
# library(episomer)
# detect_loop("data_dir")

## ----echo=FALSE, fig.cap="", out.width = '50%'--------------------------------
knitr::include_graphics("https://raw.githubusercontent.com/EU-ECDC/episomer/main/img/bluesky_authentication.png")

## ----echo=FALSE, fig.cap="", out.width = '100%'-------------------------------
knitr::include_graphics("https://raw.githubusercontent.com/EU-ECDC/episomer/main/img/topics.png")

## ----echo=FALSE, fig.cap="", out.width = '80%'--------------------------------
knitr::include_graphics("https://raw.githubusercontent.com/EU-ECDC/episomer/main/img/Example_topics_list.png")

## ----echo=FALSE, fig.cap="", out.width = '80%'--------------------------------
knitr::include_graphics("https://raw.githubusercontent.com/EU-ECDC/episomer/main/img/Schedule_plan_image.png")

## ----echo=FALSE, fig.cap="", out.width = '80%'--------------------------------
knitr::include_graphics("https://raw.githubusercontent.com/EU-ECDC/episomer/main/img/Search_plan_queues.png")

## ----echo=FALSE, fig.cap="", out.width = '80%'--------------------------------
knitr::include_graphics("https://raw.githubusercontent.com/EU-ECDC/episomer/main/img/config_languages.png")

## ----echo=FALSE, fig.cap="", out.width = '100%'-------------------------------
knitr::include_graphics("https://raw.githubusercontent.com/EU-ECDC/episomer/main/img/subscribers.png")

## ----eval=FALSE, echo=TRUE----------------------------------------------------
# episomer_app("data_dir")

## ----eval=FALSE, echo=TRUE----------------------------------------------------
# # Opens only the interactive dashboard
# dashboard_app("data_dir")
# 
# # Opens only the rest of the features with the administrative
# admin_app("data_dir")

## ----echo=FALSE, fig.cap="", out.width = '40%'--------------------------------
knitr::include_graphics("https://raw.githubusercontent.com/EU-ECDC/episomer/main/img/topics_countries.png")

## ----echo=FALSE, fig.cap="", out.width = '40%'--------------------------------
knitr::include_graphics("https://raw.githubusercontent.com/EU-ECDC/episomer/main/img/period.png")

## ----echo=FALSE, fig.cap="", out.width = '30%'--------------------------------
knitr::include_graphics("https://raw.githubusercontent.com/EU-ECDC/episomer/main/img/time_unit.png")

## ----echo=FALSE, fig.cap="", out.width = '30%'--------------------------------
knitr::include_graphics("https://raw.githubusercontent.com/EU-ECDC/episomer/main/img/include_quotes.png")

## ----echo=FALSE, fig.cap="", out.width = '30%'--------------------------------
knitr::include_graphics("https://raw.githubusercontent.com/EU-ECDC/episomer/main/img/signal_detection_fpr.png")

## ----echo=FALSE, fig.cap="", out.width = '40%'--------------------------------
knitr::include_graphics("https://raw.githubusercontent.com/EU-ECDC/episomer/main/img/outlier_dashboard.png")

## ----echo=FALSE, fig.cap="", out.width = '30%'--------------------------------
knitr::include_graphics("https://raw.githubusercontent.com/EU-ECDC/episomer/main/img/Bonferroni_correction.png")

## ----echo=FALSE, fig.cap="", out.width = '30%'--------------------------------
knitr::include_graphics("https://raw.githubusercontent.com/EU-ECDC/episomer/main/img/days_in_baseline.png")

## ----echo=FALSE, fig.cap="", out.width = '30%'--------------------------------
knitr::include_graphics("https://raw.githubusercontent.com/EU-ECDC/episomer/main/img/same_weekday_baseline.png")

## ----echo=FALSE, fig.cap="", out.width = '30%'--------------------------------
knitr::include_graphics("https://raw.githubusercontent.com/EU-ECDC/episomer/main/img/hover_trendline.png")

## ----echo=FALSE, fig.cap="", out.width = '60%'--------------------------------
knitr::include_graphics("https://raw.githubusercontent.com/EU-ECDC/episomer/main/img/hover_map.png")

## ----echo=FALSE, fig.cap="", out.width = '50%'--------------------------------
knitr::include_graphics("https://raw.githubusercontent.com/EU-ECDC/episomer/main/img/top_words.png")

## ----echo=FALSE, fig.cap="", out.width = '100%'-------------------------------
knitr::include_graphics("https://raw.githubusercontent.com/EU-ECDC/episomer/main/img/alerts.png")

## ----echo=FALSE, fig.cap="", out.width = '100%'-------------------------------
knitr::include_graphics("https://raw.githubusercontent.com/EU-ECDC/episomer/main/img/geotag_evaluation.png")

## ----echo=FALSE, fig.cap="", out.width = '100%'-------------------------------
knitr::include_graphics("https://raw.githubusercontent.com/EU-ECDC/episomer/main/img/config.png")

## ----echo=FALSE, fig.cap="", out.width = '50%'--------------------------------
knitr::include_graphics("https://raw.githubusercontent.com/EU-ECDC/episomer/main/img/status.png")

## ----echo=FALSE, fig.cap="", out.width = '70%'--------------------------------
knitr::include_graphics("https://raw.githubusercontent.com/EU-ECDC/episomer/main/img/detection_pipeline.png")

## ----echo=FALSE, fig.cap="", out.width = '50%'--------------------------------
knitr::include_graphics("https://raw.githubusercontent.com/EU-ECDC/episomer/main/img/signal_detection_config.png")

## ----echo=FALSE, fig.cap="", out.width = '40%'--------------------------------
knitr::include_graphics("https://raw.githubusercontent.com/EU-ECDC/episomer/main/img/general.png")

## ----echo=FALSE, fig.cap="", out.width = '30%'--------------------------------
knitr::include_graphics("https://raw.githubusercontent.com/EU-ECDC/episomer/main/img/region_disclaimer.png")

## ----echo=FALSE, fig.cap="", out.width = '100%'-------------------------------
knitr::include_graphics("https://raw.githubusercontent.com/EU-ECDC/episomer/main/img/topics.png")

## ----echo=FALSE, fig.cap="", out.width = '100%'-------------------------------
knitr::include_graphics("https://raw.githubusercontent.com/EU-ECDC/episomer/main/img/languages.png")

## ----echo=FALSE, fig.cap="", out.width = '100%'-------------------------------
knitr::include_graphics("https://raw.githubusercontent.com/EU-ECDC/episomer/main/img/troubleshoot.png")

## ----echo=FALSE, fig.cap="", out.width = '100%'-------------------------------
knitr::include_graphics("https://raw.githubusercontent.com/EU-ECDC/episomer/main/img/ebola.png")

