utils::globalVariables(c(
  ".data",
  "limit",
  "URLencode",
  "conf",
  "get_scala_posts_url",
  "is_secret_set",
  "last_fs_updates",
  "resp_header",
  "search_endpoint",
  "tail",
  "twitter_get",
  "iso_a2",
  "iso_a2_eh",
  "Details",
  "selected",
  "count",
  "app_error",
  "attrs",
  "browseURL",
  "cd",
  "get_token",
  "parse_plan",
  "progress_close",
  "progress_start"
))
