% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scheduler.R
\name{register_detect_runner_task}
\alias{register_detect_runner_task}
\title{Registers the alert detection task}
\usage{
register_detect_runner_task()
}
\value{
Nothing
}
\description{
registers the alert detection task or stops if no configuration has been set or if it is already running
}
\details{
Registers the alert detection task or stops if no configuration has been set or if it is already running.
To generate alerts, this task needs the post collection to had successfully run since the last time it ran.
This function will use the task scheduler on windows and will fall back to launching the runner as a separate process (attached to this session) on Linux.
}
\examples{
\dontrun{
   #getting tasks statuses
   library(episomer)
   message('Please choose the episomer data directory')
   setup_config(file.choose())
   register_detect_runner_task()
}
}
