% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shiny-app.R
\name{episomer_app}
\alias{episomer_app}
\alias{dashboard_app}
\alias{admin_app}
\title{Run the episomer Shiny app}
\usage{
episomer_app(
  data_dir = NA,
  profile = c("dashboard", "admin"),
  host = NULL,
  port = NULL
)

dashboard_app(data_dir = NA, host = NULL, port = NULL)

admin_app(data_dir = NA, host = NULL, port = NULL)
}
\arguments{
\item{data_dir}{Path to the 'data directory' containing application settings, models and collected posts.
If not provided the system will try to reuse the existing one from last session call of \code{\link{setup_config}} or use the EPI_HOME environment variable, default: NA}

\item{profile}{The profileis to run the Shiny app on, default: "dashboard" (can be "dashboard" or "admin")}

\item{host}{The IP if the network interface to run the Shiny app on, default: NULL (will run on 127.0.0.1)}

\item{port}{The port to run the Shiny app on, default: NULL (will run on a random port)}
}
\value{
The Shiny server object containing the launched application

The Shiny server object containing the launched application

The Shiny server object containing the launched application
}
\description{
Open the episomer Shiny app, used to setup the Data collection & processing pipeline, the Requirements & alerts pipeline and to visualise the outputs.
}
\details{
The episomer app is the user entry point to the episomer package. This application will help the user to setup the post collection process, manage all settings,
see the interactive dashboard visualisations, export them to Markdown or PDF, and setup the alert emails.

All its functionality is described on the episomer vignette.

The dashboard app is the user entry point to the episomer package for regular users. This application shows the interactive dashboard with visualisations, 
allow export them to Markdown or PDF.

All its functionality is described on the episomer vignette.

The admin app is contain all functionality for an administrator. It help the user to setup the post collection process, manage all settings,
and setup the alert emails.

All its functionality is described on the episomer vignette.
}
\examples{
\dontrun{
   #Running the episomer app
   library(episomer)
   message('Please choose the episomer data directory')
   setup_config(file.choose())
   episomer_app()
}
}
\seealso{
\code{\link{search_loop}}

 \code{\link{detect_loop}}
}
